import objectdraw.*;
import java.awt.*;
import javax.swing.*;
import java.applet.AudioClip;

/*
 * Main controller class for the Simon game.  Sets up the buttons and plays
 * the game, interacting with the Song class and receiving events from
 * the NoisyButtonListener.
 * 
 * Original by Williams College Computer Science faculty
 * 
 * Modified and updaed by Jim Teresco, The College of Saint Rose
 * Fall 2014, CSC 252
 * 
 * $Id: SimonController.java 2477 2014-11-10 02:54:02Z terescoj $
 * 
 */

public class SimonController extends Controller implements NoisyButtonListener {

    // the number of distinct sounds corresponding to the game buttons
    private static final int BUTTONCOUNT = 4; 
    private static final int COLORINTENSITY = 180; // how bright to make buttons
    private static final double PLAYDELAY = 1000;  // pause before playing

    // a sequence of buttons/tones that the user must reproduce
    private Song theSong;

    // create the button collecton
    private ButtonCollection theButtons = new ButtonCollection();

    // false if a song being played -- so button clicks are ignored while
    // the song is playing
    private boolean listening;

    // create the display of four buttons on the screen,
    // associate appropriate noises with those buttons,
    // create a song and start the game
    public void begin() {
        
        // buttons should appear in a grid
        getContentPane().setLayout(new GridLayout(2,2));

        // create an array of colors for the buttons
        Color shades[] = new Color[BUTTONCOUNT];
        shades[0] = new Color(COLORINTENSITY, 0, 0);
        shades[1] = new Color(0, COLORINTENSITY, 0);
        shades[2] = new Color(0, 0, COLORINTENSITY);
        shades[3] = new Color(COLORINTENSITY, COLORINTENSITY, 0);

        String noises[] = new String[BUTTONCOUNT];
        noises[0] = "C";
        noises[1] = "A";
        noises[2] = "B";
        noises[3] = "D";

        for (int buttonNum = 0; buttonNum < BUTTONCOUNT; buttonNum++) {
            NoisyButton curButton = 
                new NoisyButton(noises[buttonNum], shades[buttonNum]);
            getContentPane().add(curButton);
            curButton.addListener(this);
            theButtons.add(curButton);
        }

        // add the panel of buttons to the window
        validate();

        // start a new song and play it for the user
        theSong = new Song();
        extendAndPlay();
    }

    // add a note to the current song and play it
    private void extendAndPlay() {
        
        theSong.add(theButtons.getRandomButton());
        listening = false;
        theSong.play(this);
    }

    // invoked when the SongPlayer finished playing a song
    public void playComplete() {
        
        theSong.restart();
        listening = true;
    }

    // Check to see if the player clicked the expected button -- this is the 
    // method that is required by the NoisyButtonListener interface and will
    // be called any time someone clicks a button.
    public void noisyButtonClicked(NoisyButton theButton) {

        // we need to ignore clicks while playing the song, so only do
        // anything if listening is true.
        if (listening) {
            // the Song's getNextButton method will return the next expected
            // button to be pressed, so let's see if that button generated
            // this event
            if (theButton == theSong.getNextButton()) { 
                // player got it right -- so we flash/play that button
                theButton.flash();
            } 
            else {
                // not right, so we play the nasty sound, and start fresh with 
                // a new (empty) Song
                NoisyButton.playNastyNoise();
                theSong = new Song();
            }

            // if we have gotten to the end of the song successfully, or if we
            // just created a new empty song, we add a note and play it for the
            // user.
            if (theSong.atEnd()) {
                extendAndPlay();
            }
        }
    }
}
